#include "rheolef.h"
using namespace rheolef;
using namespace std;
#include "poiseuille.h"

int main(int argc, char**argv) {
  environment rheolef (argc, argv);
  geo  omega (argv[1]);
  space X1h (omega, "P1", "vector");
  space Bh  (omega, "bubble", "vector");
  space Qh  (omega, "P1");
  space Wh  (omega["upstream"], "P1");
  X1h.block ("wall");
  X1h.block ("upstream");
  X1h[1].block ("axis");
  X1h[1].block ("downstream");
  form a1 (X1h, X1h, "2D_D");
  form ab (Bh,  Bh,  "2D_D");
  form b1 = - form(X1h, Qh, "div");
  form bb = - form(Bh,  Qh, "div");
  form c(Bh,Bh);
  c.set_uu() = bb.uu()*diag(1./diag(ab.uu()))*trans(bb.uu());
  form mp (Qh, Qh, "mass");
  field u1h (X1h, 0);
  field ph (Qh, 0);
  string sys_coord = omega.coordinate_system_name();
  Float cr = omega.xmax()[1];
  u1h[0]["upstream"] = interpolate (Wh, u(cr,sys_coord));
  solver_abtb stokes (a1.uu(), b1.uu(), c.uu(), mp.uu());
  stokes.solve (-(a1.ub()*u1h.b()), -(b1.ub()*u1h.b()),
                u1h.set_u(),        ph.set_u());
  dout << catchmark("inv_lambda") << 0 << endl
       << catchmark("u")  << u1h
       << catchmark("p")  << ph;
}
