///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///
/// =========================================================================
#include "rheolef/d_dx0.h"

namespace rheolef {
using namespace std;
using namespace ublas;

template<class T, class M>
void 
d_dx0<T,M>::operator() (const geo_element& K, ublas::matrix<T>& m) const
{
  base::build_d_dx (K, m, 0);
}
template<class T, class M>
void
d_dx0<T,M>::initialize () const
{
  base::set_n_derivative(1);
  check_macro (
        space_constant::scalar == base::get_second_space().valued_tag(),
        "unsupported non-scalar second space for `d_dx0' form");
  check_macro (
        space_constant::scalar == base::get_first_space().valued_tag(),
        "unsupported non-scalar first space for `d_dx0' form");
}
// ----------------------------------------------------------------------------
// instanciation in library
// ----------------------------------------------------------------------------
template class d_dx0<Float,sequential>;

#ifdef _RHEOLEF_HAVE_MPI
template class d_dx0<Float,distributed>;
#endif // _RHEOLEF_HAVE_MPI

} // namespace rheolef
