#ifndef _RHEOLEF_FIELD_NONLINEAR_EXPR_V0_H
#define _RHEOLEF_FIELD_NONLINEAR_EXPR_V0_H
///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/// 
/// =========================================================================
//
// nonlinear expressions of field, as uh*vh
// there are special cases when uh is vector-valued & vh scalar
// => this is treated in field_basic::operator*
//    here all is inlined and args are converted to field_basic
//
// author: Pierre.Saramito@imag.fr
//
// date: 30 may 2012
//
#include "rheolef/field_expr.h"
#include "rheolef/field_component.h"
#include "rheolef/field_indirect.h"

namespace rheolef {

#ifdef TO_CLEAN
// ---------------------------------------------------------------------------
// uh*vh, uh/vh : there are special cases when uh is vector-valued & vh scalar
// ---------------------------------------------------------------------------
#define _RHEOLEF_field_var2_nonlinear_op(op,field_var1,field_var2)	\
template<class T, class M> \
inline \
field_basic<T,M> \
operator op (const field_var1<T,M>& uh, const field_var2<T,M>& vh) \
{ \
  return field_basic<T,M>(uh).operator op (field_basic<T,M>(vh)); \
}

#define _RHEOLEF_field_var_expr_nonlinear_op(op,field_var)		\
template<class T, class M, class Expr> \
inline \
field_basic<T,M> \
operator op (const field_var<T,M>& uh, const field_expr<Expr>& vh) \
{ \
  return field_basic<T,M>(uh).operator op (field_basic<T,M>(vh)); \
} \
template<class T, class M, class Expr> \
inline \
field_basic<T,M> \
operator op (const field_expr<Expr>& uh, const field_var<T,M>& vh) \
{ \
  return field_basic<T,M>(uh).operator op (field_basic<T,M>(vh)); \
}

#define _RHEOLEF_field_nonlinear_op(op)		\
_RHEOLEF_field_var2_nonlinear_op(op,field_basic,field_component) \
_RHEOLEF_field_var2_nonlinear_op(op,field_basic,field_component_const) \
_RHEOLEF_field_var2_nonlinear_op(op,field_basic,field_indirect) \
_RHEOLEF_field_var2_nonlinear_op(op,field_basic,field_indirect_const) \
_RHEOLEF_field_var2_nonlinear_op(op,field_component,field_basic) \
_RHEOLEF_field_var2_nonlinear_op(op,field_component,field_component) \
_RHEOLEF_field_var2_nonlinear_op(op,field_component,field_component_const) \
_RHEOLEF_field_var2_nonlinear_op(op,field_component,field_indirect) \
_RHEOLEF_field_var2_nonlinear_op(op,field_component,field_indirect_const) \
_RHEOLEF_field_var2_nonlinear_op(op,field_component_const,field_basic) \
_RHEOLEF_field_var2_nonlinear_op(op,field_component_const,field_component) \
_RHEOLEF_field_var2_nonlinear_op(op,field_component_const,field_component_const) \
_RHEOLEF_field_var2_nonlinear_op(op,field_component_const,field_indirect) \
_RHEOLEF_field_var2_nonlinear_op(op,field_component_const,field_indirect_const) \
_RHEOLEF_field_var2_nonlinear_op(op,field_indirect,field_basic) \
_RHEOLEF_field_var2_nonlinear_op(op,field_indirect,field_component) \
_RHEOLEF_field_var2_nonlinear_op(op,field_indirect,field_component_const) \
_RHEOLEF_field_var2_nonlinear_op(op,field_indirect,field_indirect) \
_RHEOLEF_field_var2_nonlinear_op(op,field_indirect,field_indirect_const) \
_RHEOLEF_field_var2_nonlinear_op(op,field_indirect_const,field_basic) \
_RHEOLEF_field_var2_nonlinear_op(op,field_indirect_const,field_component) \
_RHEOLEF_field_var2_nonlinear_op(op,field_indirect_const,field_component_const) \
_RHEOLEF_field_var2_nonlinear_op(op,field_indirect_const,field_indirect) \
_RHEOLEF_field_var2_nonlinear_op(op,field_indirect_const,field_indirect_const) \
_RHEOLEF_field_var_expr_nonlinear_op(op,field_basic) \
_RHEOLEF_field_var_expr_nonlinear_op(op,field_component) \
_RHEOLEF_field_var_expr_nonlinear_op(op,field_component_const) \
_RHEOLEF_field_var_expr_nonlinear_op(op,field_indirect) \
_RHEOLEF_field_var_expr_nonlinear_op(op,field_indirect_const) \
template<class Expr1, class Expr2> \
inline \
field_basic<typename field_expr<Expr1>::value_type, rheo_default_memory_model> \
operator op (const field_expr<Expr1>& uh, const field_expr<Expr2>& vh) \
{ \
  /* TODO: how to deduce M from field_expr<Expr> ? */ \
  typedef typename field_expr<Expr1>::value_type T; \
  typedef rheo_default_memory_model              M; \
  return field_basic<T,M>(uh).operator op (field_basic<T,M>(vh)); \
}

_RHEOLEF_field_nonlinear_op(*)
_RHEOLEF_field_nonlinear_op(/)
#undef _RHEOLEF_field_nonlinear_op
#undef _RHEOLEF_field_var2_nonlinear_op
#undef _RHEOLEF_field_var_expr_nonlinear_op
#endif // TO_CLEAN

} // namespace rheolef
#endif // _RHEOLEF_FIELD_NONLINEAR_EXPR_V0_H
