///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/// 
/// =========================================================================
#include "rheolef/field_on_pointset.h"

namespace rheolef {

template<class T, class M>
void
field_on_pointset_rep<T,M>::resize(
  const geo_basic<T,M>&       omega,
  const quadrature<T>&        quad,
  size_type                   n_comp)
{
  // ----------------------------------------------------------------------
  // size of the the array of all pointset
  // ----------------------------------------------------------------------
  size_type map_d = omega.map_dimension();
  size_type     sz = 0;
  size_type dis_sz = 0;
  for (size_type variant = reference_element::first_variant_by_dimension(map_d);
                 variant < reference_element:: last_variant_by_dimension(map_d); variant++) {
    const distributor& ige_ownership = omega.sizes().ownership_by_variant [variant];
    if (ige_ownership.dis_size() == 0) continue;
    reference_element hat_K (variant);
    _loc_nq_by_variant [variant] = quad.size (hat_K);
        sz += _loc_nq_by_variant[variant]*ige_ownership.size();
    dis_sz += _loc_nq_by_variant[variant]*ige_ownership.dis_size();
  }
  sz     *= n_comp;
  dis_sz *= n_comp;
  distributor ownership (dis_sz, omega.comm(), sz);
  base::resize (ownership);
}
template<class T, class M>
const T&
field_on_pointset_rep<T,M>::at (
        const geo_size&       gs,
        const geo_element&    K,
    	size_type             q,
    	size_type             i_comp) const
{
  // TODO: code comes from Pkd_numbering.cc => could be merged
  // get some infos on K:
  size_type variant = K.variant();
  size_type map_dim = K.dimension();
  size_type dis_ige = K.dis_ie();
  size_type iproc   = gs.ownership_by_dimension [map_dim].find_owner (dis_ige);
  if (map_dim == 0) {
    // convert node numbering to vertex numbering, for geo order > 1
    size_type dis_inod = dis_ige;
    size_type first_dis_inod = gs.node_ownership.first_index(iproc);
    size_type first_dis_iv   = gs.ownership_by_variant [variant].first_index(iproc);
    assert_macro (dis_inod >= first_dis_inod, "invalid index");
    size_type inod = dis_inod - first_dis_inod;
    size_type iv = inod;
    size_type dis_iv = first_dis_iv + iv;
    dis_ige = dis_iv;
  }
  // compute dis_igev from dis_ige:
#ifdef TO_CLEAN
  size_type first_dis_v = 0;
  for (size_type prev_variant = reference_element::first_variant_by_dimension(map_dim);
                 prev_variant < variant;
	         prev_variant++) { 
    size_type shift = gs.ownership_by_variant [prev_variant].last_index(iproc);
    first_dis_v += shift;
  }
  for (size_type next_variant = variant+1;
                 next_variant < reference_element::last_variant_by_dimension(map_dim);
	         next_variant++) { 
    size_type shift = gs.ownership_by_variant [next_variant].first_index(iproc);
    first_dis_v += shift;
  }
  assert_macro (dis_ige >= first_dis_v, "invalid index");
  size_type dis_igev = dis_ige - first_dis_v;
#endif // TO_CLEAN
  size_type dis_igev = gs.dis_ige2dis_igev (map_dim, variant, dis_ige);

  // compute the first dis_iq on K:
  size_type dis_iq0 = 0;
  for (size_type prev_variant = reference_element::first_variant_by_dimension(map_dim);
                 prev_variant < variant; 
                 prev_variant++) {
    dis_iq0 += gs.ownership_by_variant [prev_variant]. last_index(iproc) * _loc_nq_by_variant [prev_variant] * _n_comp;
  }
  dis_iq0   += dis_igev * _loc_nq_by_variant [variant] * _n_comp;

  for (size_type next_variant = variant+1;
                 next_variant < reference_element::last_variant_by_dimension(map_dim);
                 next_variant++) {
    dis_iq0 += gs.ownership_by_variant [next_variant].first_index(iproc) * _loc_nq_by_variant [next_variant] * _n_comp;
  }
  size_type dis_iq = dis_iq0 + q*_n_comp + i_comp;
  check_macro (base::ownership().is_owned (dis_iq),
	"dis_iq="<<dis_iq<< " is out of local range ["
  	<< base::ownership().first_index() << ":"
  	<< base::ownership().last_index() << "[");
  size_type iq = dis_iq - base::ownership().first_index();
  return base::operator[] (iq);
}
// ----------------------------------------------------------------------------
// instanciation in library
// ----------------------------------------------------------------------------
#define _RHEOLEF_instanciation(T,M)	 				\
template class field_on_pointset_rep<T,M>;				\
template class field_on_pointset<T,M>;					\

_RHEOLEF_instanciation(Float,sequential)
#ifdef _RHEOLEF_HAVE_MPI
_RHEOLEF_instanciation(Float,distributed)
#endif // _RHEOLEF_HAVE_MPI

} // namespace rheolef
